from pypump import PyPump
from pypump import Client
from dateutil.parser import parse
from ConfigParser import SafeConfigParser
import twitter
import os.path

def simple_verifier(url):
	print 'Please follow the instructions at the following URL:'
	print url
	return raw_input("Verifier: ") # the verifier is a string

class PumpTweetParser:
	"""Parses the ini file and provides the results on demand."""

	# This file must exist in the current directory.
	filename = 'PumpTweet.ini'


	# Parses the ini file.
	def parse_ini(self):
		print 'Reading the config file...'

		# This verifies that the ini file exists.
		if not os.path.isfile(self.filename):
			message = self.filename + ' not found.'
			raise Exception(message)

		parser = SafeConfigParser()
		parser.read(self.filename)
		self._parser = parser
		self._recent = parser.get('history', 'recent')

		# Converts the date to a usable form.
		date = parser.get('history', 'published')
		self._published = parse(date)

		# Notes whether history exists in the ini file.
		self._history = len(date) > 0 and len(self._recent) > 0

	# Logs in to the Pump server.
	def pump_login(self):
		print 'Logging into the Pump server...'

		username = self._parser.get('pump', 'username')
		key = self._parser.get('pump', 'key')
		secret = self._parser.get('pump', 'secret')
		token = self._parser.get('pump', 'token')
		token_secret = self._parser.get('pump', 'token_secret')

		client = Client(
			webfinger = username,
			name = "Pump.io",
			type = "native",
			key = key,
			secret = secret)

		pump = PyPump(
			client = client,
			token = token,
			secret = token_secret,
			verifier_callback = simple_verifier)

		me = pump.Person(username)
		
		self._username = username
		self._pump = pump
		self._me = me

	# Logs in to Twitter.
	def twitter_login(self):
		print 'Logging into Twitter...'
		key = self._parser.get('twitter', 'key')
		secret = self._parser.get('twitter', 'secret')
		token = self._parser.get('twitter', 'token')
		token_secret = self._parser.get('twitter', 'token_secret')
		api = twitter.Api(
			consumer_key=key,
			consumer_secret=secret,
			access_token_key=token,
			access_token_secret=token_secret
		)
		
		self._api = api

	def __init__(self):
		self.parse_ini()
		self.pump_login()
		self.twitter_login()

	# Writes the latest update Pump ID in the ini file.
	# Be careful when changing this.  It rewrites the ini file.
	def update_recent(self, latest, published):
		self._parser.set('history', 'recent', latest)
		self._parser.set('history', 'published', str(published))

		with open(self.filename, 'wb') as inifile:
			self._parser.write(inifile)

	# Returns the ID for the last update (from the ini file).
	def get_recent(self):
		return self._recent

	# Returns the datetime of the last update (from the ini file).
	def get_published(self):
		return self._published

	# Returns True iff there is valid history (from the ini file).
	def get_history(self):
		return self._history

	# Returns the Pump user object.
	def get_pump_me(self):
		return self._me

	# Returns the Twitter user object.
	def get_twitter_api(self):
		return self._api

	# Returns the pump username.
	def get_pump_username(self):
		return self._username



#!/usr/bin/env python
#
#   shorturl.py
#   Shorts a long URL from command line, using ur1.ca.
#   Shows original page's title and short url.
#
#   ksaver, (at identi.ca); Sep, 2010.
#   version 0.2, Mar 2011.
#   Requires BeautifulSoup library in order to work properly:
#   http://www.crummy.com/software/BeautifulSoup/
#
#   Public Domain Code.

import sys
import urllib2

from BeautifulSoup import BeautifulSoup
from urllib  import urlencode

def shorten(longurl):
    shortener = 'http://ur1.ca/'
    urlparam = {'longurl': longurl}
    encparam = urlencode(urlparam)
    request  = urllib2.Request(shortener, encparam)
    htmlpage = urllib2.urlopen(request).read()
    htmlsoup = BeautifulSoup(htmlpage)
    txtmatch = htmlsoup.p.text.find('http')
    shorturl = htmlsoup.p.text[txtmatch:]
    return shorturl

def main(url):
    htmldoc = urllib2.urlopen(url).read()
    mysoup  = BeautifulSoup(htmldoc)
    title   = mysoup.title.text
    shorturl = shorten(url)
 
    print "'%s': %s\n" % (title, shorturl)
 
if __name__ == '__main__':
    if len(sys.argv) > 1:
        url = sys.argv[1]
    else:
        url = raw_input("URL to short: ")
    main(url)



#!/usr/bin/env python
# coding=utf-8

import argparse
from pumptweet import PumpTweet

if __name__ == "__main__":
	description = 'Cross-post from pump to twitter.  Call with no options for normal use.'
	test_help = 'do a test-run without tweeting'
	parser = argparse.ArgumentParser(description=description)
	parser.add_argument('-t', '--test', help=test_help, action='store_true')
	args = parser.parse_args()
	pumptweet = PumpTweet.PumpTweet()
	
	if args.test:
		pumptweet.pull_and_test()
	else:
		pumptweet.pull_and_push()



# coding=utf-8

from PumpLogin import PumpTweetParser
from pypump import PyPump
from MLStripper import strip_tags
from shorturl import shorten
from unicodedata import normalize
from pypump.models.collection import Public
import argparse

class PumpTweet(object):
	"""Cross-posts from Pump.io to Twitter."""

	# Connect to servers.
	def connect_to_servers(self):
		# Run the parser and grab useful values.
		self.ptp = PumpTweetParser()
		self.pump_me = self.ptp.get_pump_me()
		self.pump_username = self.ptp.get_pump_username()
		self.twitter_api = self.ptp.get_twitter_api()
	
	# Returns true if the activity is public.
	def ispublic(self, activity):
		recipients = []
		recipients += getattr(activity, 'to', [])
		recipients += getattr(activity, 'cc', [])
	
		for recipient in recipients:
			if recipient.id == Public.ENDPOINT:
				return True
		
		return False
	
	# Returns recent outbox activities.
	# If testing, don't stop at recent activity.
	def get_new_activities(self, testing=False):
		print 'Looking at Pump outbox activity...'
		
		# Some of this can be replaced by 'since' if later implemented by PyPump.
		published = self.ptp.get_published()
		recent = self.ptp.get_recent()
		outbox = self.pump_me.outbox
		history = self.ptp.get_history()
	
		# Users with a lot of non-note activity might raise this.
		count = 20
	
		# The maximum number of notes to post at a time.
		# Posting too frequently might lead to errors on Twitter.
		# If this number is too small, consider a more frequent cronjob.
		allowable_posts = 3
		notes = []
	
		for activity in outbox.major[:count]:
			print '> ' + activity.obj.objectType + ' (' + str(activity.published) + ')'
	
			# Stop looking at the outbox upon finding old activity.
			if history and not testing:
				if recent == activity.id: break
				if published >= activity.published: break
	
			# Only post several notes. Others are forgotten.
			if len(notes) >= allowable_posts: break
	
			# Only post public notes.
			if not self.ispublic (activity): break
	
			obj = activity.obj
	
			# Only post notes to Twitter.
			if obj.objectType != 'note': break
	
			# Skip deleted notes.
			if obj.deleted: break
	
			# Omit posts written by others and then shared.
			note_author = obj.author.id[len('acct:'):]
			if note_author != self.pump_username: break

			notes.append(obj)
		return notes
	
	# Make the text for a tweet that includes the contest of the note.
	def make_tweet(self, note):
		max_length = 136
		content = note.content
	
		# Convert or remove HTML.
		content = strip_tags(content)
	
		# If the note fits in a tweet, post it as-is.
		# Tweets of notes this short don't need to link to their source.
		if len(content.splitlines()) == 1 and len(content) <= max_length:
			return content
	
		# Make a short URL pointing to the original note.
		# Replace the middle of the private note URL to get a usable public link.
		private_url = note.id
		short_username = self.pump_username.split('@')[0]
		public_url = private_url.replace('/api/note/', '/' + short_username + '/note/')
		short_url = shorten(public_url)
	
		# Make room for the ellipsis and URL at the end of the tweet.
		cropped_length = max_length - len(short_url) - 2
	
		# Keep only the first line.
		content = content.splitlines()[0]
		content = content[:cropped_length]
		content = content.rstrip()
	
		tweet = content + u'… ' + short_url
		return tweet
	
	# Converts posts to tweets.
	def make_tweets(self, notes):
		tweets = []
		for note in notes:
			tweets.append(self.make_tweet(note))
		return tweets
	
	# Prints a list of tweets.
	def print_tweets(self, tweets):
		print 'Printing tweets...'
		for tweet in tweets:
			normal = normalize('NFKD', tweet).encode('ascii', 'ignore')
			print '> ' + normal
	
	# Posts a list of tweets.
	def post_tweets(self, tweets):
		print 'Posting to Twitter...'
		print 'New tweet count: ' + str(len(tweets)) + '.'
		for tweet in tweets:
			self.twitter_api.PostUpdate(tweet)
	
	# Updates the ini file with the most recent entry.
	def update_recent(self):
		print 'Updating history...'
		activity = self.pump_me.outbox.major[0]
		latest = activity.id
		published = activity.published
		self.ptp.update_recent(latest, published)
	
	# Pulls from Pump and produces text for some tweets.
	# Nothing is sent to Twitter. This is for testing.
	def pull_and_test(self):
		print 'Testing PumpTweet...'
		self.connect_to_servers()
		notes = self.get_new_activities(testing=True)
		tweets = self.make_tweets(notes)
		self.print_tweets(tweets)
	
	# Pulls from Pump and pushes to Twitter.
	def pull_and_push(self):
		self.connect_to_servers()
		notes = self.get_new_activities()
		tweets = self.make_tweets(notes)
		self.print_tweets(tweets)
		self.post_tweets(tweets)
		self.update_recent()



# coding=utf-8

from HTMLParser import HTMLParser

# Class for stripping HTML from text.
class MLStripper(HTMLParser):

	def __init__(self):
		self.reset()
		self.fed = []

	def handle_data(self, d):
		self.fed.append(d)

	def get_data(self):
		return ''.join(self.fed)

# Replaces HTML entities with actual characters.
def replace_entities(html):
	unifiable = [
		('&amp;', '&'), 
		('&nbsp;', ' '),
		('&#39;', "'"),
		('&quot;', "'"),
		('&ndash;', '-'),
		('&mdash;', u'–'),
		('&rarr;', u'→'),
		('&larr;', u'←'),
		('&lrarr;', u'↔'),
		('&ldquo;', '"'),
		('&rdquo;', '"'),
	]

	for (entity, char) in unifiable:
		html = html.replace(entity, char)

	return html

# Strips tags from HTML, returning regular text.
def strip_tags(html):
	html = replace_entities(html)
	s = MLStripper()
	s.feed(html)
	return s.get_data()


#!/usr/bin/env python
# setup.py

from setuptools import setup

setup(name='pumptweet',
	version='1.5.0',
	description='Cross posts from Pump.io to Twitter.',
	setup_requires=['setuptools-markdown'],
	long_description_markdown_filename='README.md',
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Intended Audience :: Developers',
		'Intended Audience :: End Users/Desktop',
		'License :: OSI Approved',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Operating System :: MacOS :: MacOS X',
		'Operating System :: Microsoft :: Windows',
		'Operating System :: POSIX',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.7',
		'Topic :: Communications',
	],
	url='http://github.com/dper/pumptweet',
	author='Douglas Paul Perkins',
	author_email='contact@dperkins.org',
	license='MIT',
	packages=['pumptweet'],
	install_requires=[
		'pypump >= 0.5',
		'python-twitter >= 2.0',
		'BeautifulSoup >= 3.2.1',
	],
	include_package_data=True,
	scripts=[
		'pt.py',
		'pt.sh',
	],
	zip_safe=False)
